/***************************************************************************
                          arrowitem.cpp  -  description
                             -------------------
    begin                : Sat Apr 14 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QPainter>
#include <KConfigGroup>
#include "arrowitem.h"
#include "kplscene.h"

ArrowItem::ArrowItem(double x0, double y0, unsigned color,
                     QGraphicsItem* parent, double length, bool visible,
                     bool doubleHeaded, Symbol symbol, double relSize,
                     double dir) :
 VectorItem(color, x0, y0, length, dir, relSize),
 QAbstractGraphicsShapeItem(parent), m_doubleHeaded(doubleHeaded),
 m_symbol(symbol)
{
  if (!visible)
    hide();
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

ArrowItem::ArrowItem(const ArrowItem& a) :
 VectorItem(a), QAbstractGraphicsShapeItem(), m_doubleHeaded(a.m_doubleHeaded),
 m_symbol(a.m_symbol), l2(a.l2), lHead(a.lHead), head(a.head)
{
  setZValue(a.zValue());
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
  setSelected(a.isSelected());
  setVisible(a.isVisible());
}

ArrowItem::ArrowItem(const KConfigGroup* plo, unsigned color,
                     QGraphicsItem* parent) :
 VectorItem(plo, color), QAbstractGraphicsShapeItem(parent)
{
  if (!plo->readEntry("active", true))
    hide();
  m_doubleHeaded = plo->readEntry("double", false);
  m_symbol = Symbol(plo->readEntry("symb", int(SolidLine)));
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

ArrowItem::~ArrowItem()
{
}

QRectF ArrowItem::boundingRect() const
{
  double dx = lHead;
  double dy = 0.2 * lHead;
  if (m_doubleHeaded)
    dx = l2;
  else
    if (l2 > dx)
      dx = l2 + 0.5 * pen().widthF();
  return QRectF(-dx, -dy, dx, dy + dy);
}

void ArrowItem::paint(QPainter* painter, const QStyleOptionGraphicsItem*,
                      QWidget*)
{
  if (isSelected()) {
    painter->setPen(QPen(Qt::DashLine));
    painter->drawRect(boundingRect());
  }
  if (l2 > (m_doubleHeaded ? lHead + lHead : lHead)) {
    painter->setPen(pen());
    painter->drawLine(m_doubleHeaded ? -l2 + lHead : -l2, 0, -lHead, 0);
  }
  painter->setPen(Qt::NoPen);
  painter->setBrush(brush());
  painter->drawConvexPolygon(head);
  if (m_doubleHeaded) {
    painter->rotate(180);
    painter->translate(l2, 0);
    painter->drawConvexPolygon(head);
  }
}

QPixmap ArrowItem::pixmap() const
{
  return KplItem::pixmap(pen(), m_symbol);
}

void ArrowItem::setHead()
{
  lHead = parentWidth(this) * m_relSize / 30;
  double dy = 0.2 * lHead;
  head.clear();
  head << QPointF() << QPointF(-lHead, dy) << QPointF(-lHead, -dy);
}

void ArrowItem::setPenWidth()
{
  QPen p = pen();
  double w = penWidth(this);
  if (p.widthF() != w) {
    prepareGeometryChange();
    setPen(changePenWidth(p, w));
    setHead();
  }
}

void ArrowItem::setProperties(const QTransform& m)
{
  m_matrix = m;
  setProperties();
}

void ArrowItem::update()
{
  prepareGeometryChange();
  setProperties();
  QGraphicsItem::update();
}

void ArrowItem::writePlo(KConfigGroup* plo) const
{
  plo->writeEntry("Type", "ARROWITEM");
  VectorItem::writePlo(plo, isVisible());
  plo->writeEntry("double", m_doubleHeaded);
  plo->writeEntry("symb", int(m_symbol));
}

void ArrowItem::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
  QAbstractGraphicsShapeItem::mouseReleaseEvent(event);
  ((KplScene*)scene())->updatePos(this);
}

void ArrowItem::setProperties()
{
  setPos(m_matrix.map(QPointF(m_x0, m_y0)));
  setBrush(createBrush(m_color, SolidPattern));
  setPen(createPen(this, m_color, FramePen, m_symbol));
  setRotation(-m_dir);
  setHead();
  l2 = m_matrix.m11() * m_length;
}
