//
// C++ Interface: calc3djob
//
// Description: 
//
//
// Author: Werner Stille <kpl@werner-stille.de>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef CALC3DJOB_H
#define CALC3DJOB_H

#include "calcjob.h"

/**
  * Calculation 3D job class. Class for calculation of 3D function value tables
  * by multithreading.
  * @author Werner Stille
  */
class Calc3DJob : public CalcJob
{
public:
  /**
    * Constructor.
    * @param x array with x argument values.
    * @param z array with z argument values.
    * @param values array for function values.
    * @param func function.
    * @param par parameter array.
    * @param ix0 start index for x values.
    * @param nx number of x values.
    * @param nz number of z values.
    */
  Calc3DJob(const double* x, const double* z, double** values,
            double (*func)(double, double, const double*),
            const double* par, int ix0, int nx, int nz);
  /** Destructor. */
  ~Calc3DJob();

protected:
  /** Calculates function values. */
  void run(ThreadWeaver::JobPointer, ThreadWeaver::Thread*);
  const double* m_x;
  double** m_values3D;
  double (*m_func3D)(double, double, const double*);
  int m_nx, m_ix0;
};

#endif
