/***************************************************************************
                          fun3ditem.cpp  -  description
                             -------------------
    begin                : Tue Dec 25 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QLibrary>
#include <QElapsedTimer>
#include <KConfigGroup>
#define TRANSLATION_DOMAIN "libkpl"
#include <KLocalizedString>
#include <KMessageBox>
#include "arrayitem.h"
#include "calc3djob.h"
#include "fun3ditem.h"
#include "utils.h"

using namespace ThreadWeaver;

Fun3DItem::Fun3DItem(double x0, double y0, double width, double height,
                     unsigned colorFrame, unsigned colorGrid,
                     unsigned colorData, bool logX, bool logY, bool logZ,
                     GridMode gridMode, Mode3D mode3D, double xMax,
                     double yMax, double zMax, const QString& textX,
                     const QString& textY, const QString& textZ,
                     const QString& header, double relSize, int numDigX,
                     int numDigY, int numDigZ, int minTicX, int minTicY,
                     int minTicZ, int expX, int expY, int expZ, double xMin,
                     double yMin, double zMin, double xTic, double yTic,
                     double zTic, double deltaX, double deltaZ, double phi,
                     double theta, double bx, double by, double bz,
                     double xr, double yr, double fx, double fy, double fz,
                     const QString& name, const QUrl& url, bool showFrame,
                     bool collapsed, bool visible) :
 ThreeDItem(x0, y0, width, height, colorFrame, colorGrid, colorData, logX,
            logY, logZ, gridMode, mode3D, xMax, yMax, zMax, textX, textY,
            textZ, header, relSize, numDigX, numDigY, numDigZ, minTicX,
            minTicY, minTicZ, expX, expY, expZ, xMin, yMin, zMin,
            xTic, yTic, zTic, deltaX, deltaZ, phi, theta, bx, by, bz, xr, yr,
            fx, fy, fz, showFrame, collapsed, visible),
 funcY(0), funcYo(0), m_nameY(name), m_urlY(url), m_libY(0)
{
  memset(m_parY, 0, sizeof(m_parY));
  memset(parYo, 0, sizeof(parYo));
  if (!m_urlY.url().isEmpty())
    Utils::getFuncAddr(m_urlY.path(), m_nameY, &m_libY,
                       (QFunctionPointer*) &funcY);
}

Fun3DItem::Fun3DItem(const Fun3DItem& f) :
 ThreeDItem(f), funcY(f.funcY), funcYo(f.funcYo), m_nameY(f.m_nameY),
 m_urlY(f.m_urlY), m_libY(0)
{
  memcpy(m_parY, f.m_parY, sizeof(m_parY));
  memcpy(parYo, f.parYo, sizeof(parYo));
  if (f.m_libY)
    Utils::getFuncAddr(m_urlY.path(), m_nameY, &m_libY,
                       (QFunctionPointer*) &funcY);
}

Fun3DItem::Fun3DItem(const KConfigGroup* plo, const QUrl& urlPlo, double x0,
                     double y0, double width, double height,
                     unsigned colorFrame, unsigned colorGrid,
                     unsigned colorData, GridMode gridMode) :
 ThreeDItem(plo, x0, y0, width, height, colorFrame, colorGrid, colorData,
            gridMode), funcYo(0), m_libY(0)
{
  memset(parYo, 0, sizeof(parYo));
  FunItem::stringToPar(plo->readEntry("p"), m_parY);
  m_nameY = plo->readEntry("name", "");
  m_urlY = absoluteUrl(plo->readEntry("path", ""), urlPlo);
  Utils::getFuncAddr(m_urlY.path(), m_nameY, &m_libY, (QFunctionPointer*) &funcY);
}

Fun3DItem::~Fun3DItem()
{
  unload();
}

int Fun3DItem::calcTable() const
{
  if (!funcY)
    return 0;
  bool newv = !m_yValues;
  if (m_deltaX) {
    nx = int(fabs((m_xMax - m_xMin) / m_deltaX)) + 1;
    xStep = m_logX ? (log10(m_xMax / m_xMin) / (nx - 1)) : m_deltaX;
  } else {
    nx = 41;
    xStep = 0.025 * (m_logX ? log10(m_xMax / m_xMin) : (m_xMax - m_xMin));
  }
  if (m_deltaZ) {
    nz = int(fabs((m_zMax - m_zMin) / m_deltaZ)) + 1;
    zStep = m_logZ ? (log10(m_zMax / m_zMin) / (nz - 1)) : m_deltaZ;
  } else {
    nz = 41;
    zStep = 0.025 * (m_logZ ? log10(m_zMax / m_zMin) : (m_zMax - m_zMin));
  }
  int n = nx * nz;
  bool pchanged = false;
  for (int i = 0; i < FunItem::NumParMax; ++i)
    if ((pchanged = (m_parY[i] != parYo[i])))
      break;
  if (pchanged || newv || (m_xMin != xMino) || (m_xMax != xMaxo) ||
      (m_deltaX != deltaXo) || (m_logX != logXo) || (m_zMin != zMino) ||
      (m_zMax != zMaxo) || (m_deltaZ != deltaZo) || (m_logZ != logZo) ||
      (funcY != funcYo)) {
    ArrayItem::freeArray(&m_yValues);
    double* y1 = new double[n];
    m_yValues = new double*[nx];
    double xa[nx];
    for (int ix = 0; ix < nx; ++ix) {
      m_yValues[ix] = &y1[ix * nz];
      xa[ix] = m_logX ? pow(10, log10(m_xMin) + ix * xStep) :
                        (m_xMin + ix * xStep);
    }
    double za[nz];
    for (int iz = 0; iz < nz; ++iz)
      za[iz] = m_logZ ? pow(10, log10(m_zMin) + iz * zStep) :
                        (m_zMin + iz * zStep);
    int nJobs =
      qMin(nx, Queue::instance()->maximumNumberOfThreads());
    QElapsedTimer timer;
    timer.start();
    if (nJobs > 1) {
      int ix0 = 0;
      for (int i = 0; i < nJobs; ++i) {
        int nt = (nx - ix0) / (nJobs - i);
        stream() << new Calc3DJob(xa, za, m_yValues, funcY, m_parY, ix0, nt,
                                  nz);
        ix0 += nt;
      }
      Queue::instance()->finish();
    } else
      for (int ix = 0; ix < nx; ++ix)
        for (int iz = 0; iz < nz; ++iz)
          m_yValues[ix][iz] = ((*funcY)(xa[ix], za[iz], m_parY));
    long tns = timer.nsecsElapsed();
    qDebug("Fun3DItem:calcTable(): %d points,  %d threads, %4.4g s, "
           "%4.4g points / s", n, nJobs, 1e-9 * tns, 1e9 * n / tns);
  }
  if (pchanged)
    for (int i = 0; i < FunItem::NumParMax; ++i)
      parYo[i] = m_parY[i];
  funcYo = funcY;
  xMino = m_xMin;
  xMaxo = m_xMax;
  deltaXo = m_deltaX;
  logXo = m_logX;
  zMino = m_zMin;
  zMaxo = m_zMax;
  deltaZo = m_deltaZ;
  logZo = m_logZ;
  return n;
}

void Fun3DItem::exportTable(QTextStream& stream, char separator, char format,
                            int precision) const
{
  calcTable();
  ThreeDItem::exportTable(stream, separator, format, precision);
}

void Fun3DItem::paint(QPainter* painter,
                      const QStyleOptionGraphicsItem* option,
                      QWidget* widget)
{
  calcTable();
  ThreeDItem::paint(painter, option, widget);
}

bool Fun3DItem::setFuncY(const QUrl& url, const QString& name)
{
  m_urlY = url;
  m_nameY = name;
  unload();
  return Utils::getFuncAddr(url.path(), name, &m_libY,
                            (QFunctionPointer*) &funcY);
}

void Fun3DItem::update()
{
  calcTable();
  ThreeDItem::update();
}

void Fun3DItem::writePlo(KConfigGroup* plo, const QUrl& urlPlo, bool absolute,
                         char format, int precision) const
{
  ThreeDItem::writePlo(plo);
  plo->writeEntry("Type", "FUN3DITEM");
  plo->writeEntry("xmi", QString::number(m_xMin, format, precision));
  plo->writeEntry("xma", QString::number(m_xMax, format, precision));
  plo->writeEntry("zmi", QString::number(m_zMin, format, precision));
  plo->writeEntry("zma", QString::number(m_zMax, format, precision));
  plo->writeEntry("dx", QString::number(m_deltaX, format, precision));
  plo->writeEntry("dz", QString::number(m_deltaZ, format, precision));
  plo->writeEntry("p", FunItem::parToString(m_parY, format, precision));
  plo->writeEntry("name", m_nameY);
  plo->writeEntry("path", path(urlPlo, m_urlY, absolute));
}

void Fun3DItem::unload()
{
  if (m_libY) {
    m_libY->unload();
    delete m_libY;
    m_libY = 0;
  }
}
