/***************************************************************************
                          lineitem.h  -  description
                             -------------------
    begin                : Mon Apr 16 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LINEITEM_H
#define LINEITEM_H

#include <QGraphicsLineItem>
#include "vectoritem.h"

class KConfigGroup;

/**
  * Line item class. Class for line decoration items.
  * @author Werner Stille
  */
class Q_DECL_EXPORT LineItem : public VectorItem, public QGraphicsLineItem
{
public:
  /** Specific QGraphicsItem type number for function type() */
  enum {Type = UserType + Line
    /**< Specific QGraphicsItem type number of LineItem for function
     * type().
     */
  };
  /**
    * Constructor. Initializes corresponding to arguments.
    * @param x0 x start position in cm.
    * @param y0 y start position in cm.
    * @param color color for line.
    * @param parent pointer to parent item.
    * @param length length in cm.
    * @param visible true for visible item.
    * @param symbol line type.
    * @param relSize relative line width.
    * @param dir direction in degree.
    */
  LineItem(double x0 = 0, double y0 = 0, unsigned color = 0,
           QGraphicsItem* parent = 0, double length = 1, bool visible = true,
           Symbol symbol = SolidLine, double relSize = 1, double dir = 0);
  /**
    * Copy constructor.
    * @param a source item.
    */
  LineItem(const LineItem& a);
  /**
    * Constructor. Initializes corresponding to plot file.
    * @param plo pointer to KSimpleConfig object containing item data.
    * @param color default color for line.
    * @param parent pointer to parent item.
    */
  LineItem(const KConfigGroup* plo, unsigned color = 0,
           QGraphicsItem* parent = 0);
  /** Destructor. */
  virtual ~LineItem();
  /** Returns decoration pixmap for item list. */
  QPixmap pixmap() const;
  /** Sets offset for dashed lines. */
  void setDashOffset();
  /** Sets pen width. */
  void setPenWidth();
  /**
    * Sets properties according to transformation matrix.
    * @param m transformation matrix.
    */
  void setProperties(const QTransform& m);
  /**
    * Sets line type.
    * @param symbol line type.
    */
  inline void setSymbol(Symbol symbol) { m_symbol = symbol; };
  /** Returns line type. */
  inline Symbol symbol() const { return m_symbol; };
  /** Returns specific QGraphicsItem type number. */
  virtual int type() const { return Type; };
  /** Applies changes and forces redrawing. */
  void update();
  /**
   * Writes into plot file.
   * @param plo KConfigGroup object.
   */
  void writePlo(KConfigGroup* plo) const;

protected:
  /** Updates position. */
  virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent* event);
  /** Sets properties according to transformation matrix. */
  void setProperties();
  double myDashOffset;
  Symbol m_symbol;
};

#endif
