/***************************************************************************
                          arcdlg.cpp  -  description
                             -------------------
    begin                : Sat Aug 17 2002
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QCheckBox>
#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QGridLayout>
#include <QPushButton>
#include <QUndoStack>
#include <KLocalizedString>
#include "arcdlg.h"
#include "arcitem.h"
#include "kpldoc.h"
#include "spinbox.h"
#include "symboldlg.h"

ArcDlg::ArcDlg(QWidget* parent, KplDoc* model, ArcItem* ai0,
               const QPoint& pos) :
 DecoDlg(parent, i18n("Arc"), model, ai0, pos), ai(ai0)
{
  grid->addWidget(new QLabel(i18n("Width")), 0, 0);
  grid->addWidget(eWidth = new QDoubleSpinBox, 0, 1);
  eWidth->setRange(0.1, 200);
  eWidth->setSingleStep(0.1);
  eWidth->setSuffix(" cm");
  eWidth->setValue(ai->width());
  connect(eWidth, SIGNAL(valueChanged(const QString&)), SLOT(slotAspect()));
  grid->addWidget(new QLabel(i18n("Height")), 0, 3);
  grid->addWidget(eHeight = new QDoubleSpinBox, 0, 4);
  eHeight->setRange(0.1, 200);
  eHeight->setSingleStep(0.1);
  eHeight->setSuffix(" cm");
  eHeight->setValue(ai->height());
  grid->addWidget(isotropic = new QCheckBox(i18n("Circular arc")), 1, 1, 1, 5);
  connect(isotropic, SIGNAL(toggled(bool)), SLOT(slotAspect()));
  isotropic->setChecked(ai->width() == ai->height());
  grid->addWidget(new QLabel(i18n("Start")), 2, 0);
  grid->addWidget(eStart = new QDoubleSpinBox, 2, 1);
  eStart->setRange(-360, 360);
  eStart->setSuffix(QString::fromUtf8("\302\260"));
  eStart->setValue(ai->start());
  grid->addWidget(new QLabel(i18n("Length")), 2, 3);
  grid->addWidget(eSpan = new QDoubleSpinBox, 2, 4);
  eSpan->setRange(-360, 360);
  eSpan->setSuffix(QString::fromUtf8("\302\260"));
  eSpan->setValue(ai->span());
  grid->addWidget(new QLabel(i18n("Rotation")), 3, 0);
  grid->addWidget(eDir = new QDoubleSpinBox, 3, 1);
  eDir->setRange(-360, 360);
  eDir->setSuffix(QString::fromUtf8("\302\260"));
  eDir->setValue(ai->dir());
  initEntries(4);
  grid->addWidget(new QLabel(i18n("Type")), 6, 0);
  grid->addWidget(symb = new SpinBox(ai->symbol(), 1, 9), 6, 1);
  grid->addWidget(new SymbolButton(symb, true, false), 6, 2);
  lSize->setText(i18n("Line width"));
  connect(eHeight, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eStart, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eSpan, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eDir, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(symb, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(buttonBox, SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
}

ArcDlg::~ArcDlg()
{
}

void ArcDlg::slotAspect()
{
  bool iso = isotropic->isChecked();
  if (iso)
    eHeight->setValue(eWidth->value());
  eHeight->setDisabled(iso);
  slotEdited();
}

void ArcDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
      buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
      buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    case QDialogButtonBox::Ok: {
      ArcItem* aiNew = new ArcItem(*ai);
      getValues(aiNew);
      double w = eWidth->value();
      aiNew->setWidth(w);
      aiNew->setHeight(isotropic->isChecked() ? w : eHeight->value());
      aiNew->setStart(eStart->value());
      aiNew->setSpan(eSpan->value());
      aiNew->setDir(eDir->value());
      aiNew->setSymbol(KplItem::Symbol(symb->value()));
      m->undoStack()->push(new EditCommand(aiNew, ai, m));
      ai = aiNew;
      if (id == QDialogButtonBox::Ok)
        accept();
      break;
    }
    case QDialogButtonBox::Help:
      QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/arc.html")));
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}
