/***************************************************************************
                          brushdlg.cpp  -  description
                             -------------------
    begin                : Mit Mai 14 2003
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QApplication>
#include <QButtonGroup>
#include <QDialogButtonBox>
#include <QGroupBox>
#include <QLayout>
#include <qstyle.h>
#include <QStyleOptionButton>
#include <QStylePainter>
#include <KLocalizedString>
#include "brushdlg.h"
#include "kplitem.h"

BrushButton::BrushButton(int brush, QWidget* parent) :
 QPushButton(parent), m_brush(brush)
{
  connect(this, SIGNAL(clicked()), SLOT(slotBrush()));
}

BrushButton::~BrushButton()
{
}

QSize BrushButton::sizeHint() const
{
  QStyleOptionButton opt;
  initStyleOption(&opt);
  return style()->sizeFromContents(QStyle::CT_PushButton, &opt, QSize(40, 20),
                                   this).
    expandedTo(QApplication::globalStrut());
}

void BrushButton::setValue(int brush)
{
  m_brush = brush;
  repaint();
}

int BrushButton::value()
{
  return m_brush;
}

void BrushButton::slotBrush()
{
  int oldBrush = m_brush;
  BrushDlg dlg(&m_brush, parentWidget());
  dlg.exec();
  if (m_brush != oldBrush)
    emit changed(m_brush);
}

void BrushButton::paintEvent(QPaintEvent*)
{
  QPainter p(this);
  QStyleOptionButton opt;
  initStyleOption(&opt);
  style()->drawControl(QStyle::CE_PushButtonBevel, &opt, &p, this);
  QRect r = style()->subElementRect(QStyle::SE_PushButtonContents, &opt, this);
  r.adjust(1, 1, -1, -2);
  if (m_brush)
    p.fillRect(r, KplItem::createBrush(0, KplItem::FillStyle(m_brush)));
  else
    p.drawRect(r);
}

BrushRadioButton::BrushRadioButton(int brush, QWidget* parent) :
 QRadioButton(parent), m_brush(brush)
{
}

BrushRadioButton::~BrushRadioButton()
{
}

void BrushRadioButton::paintEvent(QPaintEvent*)
{
  QPainter p(this);
  QStyleOptionButton opt;
  initStyleOption(&opt);
  style()->drawControl(QStyle::CE_RadioButton, &opt, &p, this);
  QRect r = style()->subElementRect(QStyle::SE_RadioButtonContents, &opt, this);
  r.adjust(1, 1, -1, -2);
  if (m_brush)
    p.fillRect(r, KplItem::createBrush(0, KplItem::FillStyle(m_brush)));
  else
    p.drawRect(r);
}

BrushDlg::BrushDlg(int* brush, QWidget* parent) :
 QDialog(parent), m_brush(brush)
{
  setWindowTitle(i18n("Patterns"));
  QVBoxLayout* vbox = new QVBoxLayout(this);
  QGroupBox* g = new QGroupBox;
  vbox->addWidget(g);
  QGridLayout* grid = new QGridLayout;
  g->setLayout(grid);
  bg = new QButtonGroup;
  BrushRadioButton* rb;
  for (int i = 0; i < 8; i++) {
    grid->addWidget(rb = new BrushRadioButton(i), i % 4, i / 4);
    bg->addButton(rb, i);
    if (i == *m_brush)
      rb->setChecked(true);
  }
  QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok |
                                                     QDialogButtonBox::Cancel);
  vbox->addWidget(buttonBox);
  bOk =   buttonBox->button(QDialogButtonBox::Ok);
  bOk->setEnabled(false);
  connect(bg, SIGNAL(buttonClicked(int)), SLOT(slotEdited()));
  connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
  setMinimumWidth(280);
}

BrushDlg::~BrushDlg()
{
}

void BrushDlg::accept()
{
  *m_brush = bg->checkedId();
  QDialog::accept();
}

void BrushDlg::slotEdited()
{
  bOk->setEnabled(true);
}
