/***************************************************************************
                          editparameterdlg.h  -  description
                             -------------------
    begin                : Mit Jul 23 2003
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EDITPARAMETERDLG_H
#define EDITPARAMETERDLG_H

#include <QDialog>
#include "funitem.h"

class QDialogButtonBox;
class DoubleEdit;
class SourceWidget;

/**
  * Edit parameter dialog class. Provides a dialog for the editing
  * the parameters of a function.
  * @author Werner Stille
  */
class EditParameterDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param p parameter array.
   * @param file absolute path of the function library.
   * @param func function name.
   * @param applyButton true to show a button for applying.
   */
  EditParameterDlg(QWidget* parent, double* p, const QString& file,
                   const QString& func, bool applyButton = false);
  /** Destructor. */
  ~EditParameterDlg();

protected:
  double* p;
  DoubleEdit* par[FunItem::NumParMax];
  QDialogButtonBox* buttonBox;
  QString func;
  SourceWidget* sw;

signals:
  /** Emitted when the Apply button was pressed. */
  void applyClicked();

protected slots:
  /** Highlights function name in source widget. */
  void highlight();
  /**
   * Handles button clicks.
   * @param button pointer to button.
   */
  virtual void slotButtonClicked(QAbstractButton* button);
  /** Enables buttons. */
  void slotEdited();
};

#endif
