/***************************************************************************
                          edittextdlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EDITTEXTDLG_H
#define EDITTEXTDLG_H

#include <QDialog>
#include <QTextDocument>

class QDialogButtonBox;
class QLineEdit;
class QToolButton;

/**
  * Text preview widget class. Provides a widget for the
  * previewing of text containing subscripts, superscripts, and several fonts.
  * @author Werner Stille
  */
class Preview : public QWidget
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   */
  Preview(QWidget* parent = 0);
  /** Destructor. */
  virtual ~Preview();

public slots:
  /**
   * Updates the text display.
   * @param s text.
   */
  void setText(const QString& s);

protected:
  /**
   * Reimplemented for painting the text preview widget.
   */
  virtual void paintEvent(QPaintEvent*);
  QTextDocument m_doc;
};

/**
  * Text editing dialog class. Provides a dialog to edit
  * text containing subscripts, superscripts, and several fonts.
  * @author Werner Stille
  */
class EditTextDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param ed pointer to QLineEdit widget containing the text.
   */
  EditTextDlg(QWidget* parent, QLineEdit* ed);
  /** Destructor. */
  virtual ~EditTextDlg();
  /**
   * Displays dialog to edit the contents of a QLineEdit widget.
   * @param parent pointer to parent widget.
   * @param ed pointer to widget.
   */
  static void editText(QWidget* parent, QLineEdit* ed);

protected:
  /**
   * Reimplemented to display help when key F1 is pressed.
   * @param e Key event.
   */
  virtual void keyPressEvent(QKeyEvent* e);
  QString sfont;
  Preview* editPreview;
  QDialogButtonBox* buttonBox;
  QLineEdit *font, *ed, *text;
  QToolButton* bCat;

protected slots:
  /**
   * Handles button clicks.
   * @param button pointer to button.
   */
  void slotButtonClicked(QAbstractButton* button);
  /** Concatenates current text and control sequence. */
  void slotCat();
  /** Enables Ok button. */
  void slotEdited();
  /**
   * Builds control sequence with selected font.
   * @param id font button id.
   */
  void slotFont(int id);
  /**
   * Builds control sequence with selected vertical position.
   * @param id position button id.
   */
  void slotPos(int id);
  /** Restores window size. */
  void slotRestoreSize();
};

#endif
