/***************************************************************************
                          ellipsedlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ELLIPSEDLG_H
#define ELLIPSEDLG_H

#include "decodlg.h"

class QCheckBox;
class QDoubleSpinBox;
class BrushButton;
class EllipseItem;
class SymbolButton;

/**
  * Ellipse settings dialog class. Provides a dialog for the settings
  * of an ellipse decoration item.
  * @author Werner Stille
  */
class EllipseDlg : public DecoDlg
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param ei pointer to ellipse item.
   * @param pos dialog position.
   */
  EllipseDlg(QWidget* parent, KplDoc* model, EllipseItem* ei,
             const QPoint& pos);
  /** Destructor. */
  ~EllipseDlg();

protected:
  EllipseItem* ei;
  QCheckBox* isotropic;
  SpinBox* symb;
  BrushButton* fill;
  SymbolButton* bSymb;
  QDoubleSpinBox *eWidth, *eHeight, *eDir;

protected slots:
  /** Sets height widget. */
  void slotAspect();
  /**
   * Handles button clicks.
   * @param button pointer to button.
   */
  void slotButtonClicked(QAbstractButton* button);
  /**
   * Adjusts state of widgets when fill pattern is changed.
   * @param brush new pattern number.
   */
  void slotFillChanged(int brush);
};

#endif
