//
// C++ Implementation: filedialog
//
// Description: 
//
//
// Author: Werner Stille <kpl@werner-stille.de>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <QComboBox>
#include <QDialogButtonBox>
#include <QLayout>
#include <QPushButton>
#include <QTimer>
#include <KFileWidget>
#include <KLocalizedString>
#include <KWindowConfig>
#include "arrayitem.h"
#include "filedialog.h"
#include "kplsettings.h"
#include "utils.h"

FileDialog::FileDialog(const QUrl& startDir, const QString& filter,
                       ArrayItem::DecimalCharacter idec, KFile::Modes mode,
                       QWidget* parent) :
 QDialog(parent)
{
  comboBox = new QComboBox;
  comboBox->insertItems(0, QStringList() << "." << ",");
  comboBox->setCurrentIndex(idec);
  init(startDir, filter, mode, comboBox);
}

FileDialog::FileDialog(const QUrl& startDir, const QString& filter,
                       KFile::Modes mode, QWidget* parent) :
 QDialog(parent)
{
  init(startDir, filter, mode);
}

FileDialog::~FileDialog()
{
  KConfigGroup config = KSharedConfig::openConfig()->group("FileDialog");
  KWindowConfig::saveWindowSize(windowHandle(), config);
}

ArrayItem::DecimalCharacter FileDialog::decimalCharacter()
{
  return static_cast<ArrayItem::DecimalCharacter>(comboBox->currentIndex());
}

QUrl FileDialog::getReadUrl(ArrayItem::DecimalCharacter& idec,
                            const QString& startDir, QWidget* parent,
                            const QString& filter)
{
  FileDialog dlg(Utils::url(startDir), filter, idec, KFile::File, parent);
  dlg.exec();
  idec = dlg.decimalCharacter();
  return dlg.selectedUrl();
}

QList<QUrl> FileDialog::getReadUrls(ArrayItem::DecimalCharacter& idec,
                                    QWidget* parent, const QString& filter)
{
  FileDialog dlg(Utils::url(KplSettings::currentDir()), filter, idec,
                            KFile::Files, parent);
  dlg.exec();
  idec = dlg.decimalCharacter();
  return dlg.selectedUrls();
}

QList<QUrl> FileDialog::getReadUrls(QWidget* parent, const QString& filter)
{
  FileDialog dlg(Utils::url(KplSettings::currentDir()), filter, KFile::Files,
                 parent);
  dlg.exec();
  return dlg.selectedUrls();
}

QUrl FileDialog::selectedUrl() const
{
  return fileWidget->selectedUrl();
}

QList<QUrl> FileDialog::selectedUrls() const
{
  return fileWidget->selectedUrls();
}

void FileDialog::accept()
{
  fileWidget->accept();
  QDialog::accept();
}

void FileDialog::slotRestoreSize()
{
  KWindowConfig::restoreWindowSize(windowHandle(),
    KSharedConfig::openConfig()->group("FileDialog"));
}

void FileDialog::init(const QUrl& startDir, const QString& filter,
                      KFile::Modes mode, QWidget* customWidget)
{
  setWindowTitle(i18n("Open"));
  QVBoxLayout* vbox = new QVBoxLayout(this);
  fileWidget = new KFileWidget(startDir, this);
  fileWidget->setMode(mode);
  fileWidget->setFilter(filter);
  if (customWidget)
    fileWidget->setCustomWidget(i18n("Decimal symbol:"), customWidget);
  vbox->addWidget(fileWidget);
  connect(fileWidget, SIGNAL(accepted()), SLOT(accept()));
  QDialogButtonBox* buttonBox = new QDialogButtonBox;
  vbox->addWidget(buttonBox);
  QPushButton* b = fileWidget->okButton();
  buttonBox->addButton(b, QDialogButtonBox::AcceptRole);
  connect(b, SIGNAL(clicked(bool)), fileWidget, SLOT(slotOk()));
  b = fileWidget->cancelButton();
  buttonBox->addButton(b, QDialogButtonBox::RejectRole);
  connect(b, SIGNAL(clicked(bool)), SLOT(reject()));
  QTimer::singleShot(0, this, SLOT(slotRestoreSize()));
}

