/***************************************************************************
                          imagedlg.h  -  description
                             -------------------
    begin                : Don Jun 5 2003
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IMAGEDLG_H
#define IMAGEDLG_H

#include <QDialog>

class QCheckBox;
class QDoubleSpinBox;
class ImageItem;
class KUrlRequester;
class KplDoc;

/**
  * Image settings dialog class. Provides a dialog for the settings
  * of an EPS image item.
  * @author Werner Stille
  */
class ImageDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param ii pointer to image item.
   * @param pos dialog position.
   */
  ImageDlg(QWidget* parent, KplDoc* model, ImageItem* ii, const QPoint& pos);
  /** Destructor. */
  ~ImageDlg();

protected:
  /**
   * Reimplemented to display help when key F1 is pressed.
   * @param e Key event.
   */
  virtual void keyPressEvent(QKeyEvent* e);
  KplDoc* m;
  ImageItem* ii;
  QDialogButtonBox* buttonBox;
  KUrlRequester* fileName;
  QDoubleSpinBox *ex0, *ey0, *efx, *efy, *eDir;
  QCheckBox* isotropic;

protected slots:
  /** Sets y scaling widget. */
  void slotAspect();
  /**
   * Handles button clicks.
   * @param button pointer to button.
   */
  virtual void slotButtonClicked(QAbstractButton* button);
  /** Enables buttons. */
  void slotEdited();
  /** Restores window size. */
  void slotRestoreSize();
};

#endif
