/***************************************************************************
                          main.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999

    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QApplication>
#include <QCommandLineParser>
#include <QIcon>
#include <QLocale>
#include <QUrl>
#include <KAboutData>
#include <KLocalizedString>
#include "kpl.h"
#include "utils.h"

int main(int argc, char* argv[])
{
  QLocale::setDefault(QLocale::c());
  QApplication application(argc, argv);
  KLocalizedString::setApplicationDomain("kpl");
  KAboutData aboutData(QStringLiteral("kpl"), i18n("Kpl"),
                       QStringLiteral("5.0"),
                       i18n("An application for plotting of data sets and "
                            "functions"), KAboutLicense::GPL,
                       i18n("(C) 1999-2017, Werner Stille "
                            "<kpl@werner-stille.de>"), QString(), QString(),
                       QStringLiteral("kpl@werner-stille.de"));
  aboutData.addAuthor(i18n("Werner Stille"), QString(),
                      QStringLiteral("kpl@werner-stille.de"));
  KAboutData::setApplicationData(aboutData);
  application.setWindowIcon(QIcon::fromTheme("kpl"));
  QCommandLineParser parser;
  parser.addPositionalArgument(QStringLiteral("urls"),
                               i18n("Documents to open"), "[urls...]");
  aboutData.setupCommandLine(&parser);
  parser.process(application);
  aboutData.processCommandLine(&parser);
  if (application.isSessionRestored())
    kRestoreMainWindows<Kpl>();
  else {
    const QStringList args = parser.positionalArguments();
    Kpl* kpl = new Kpl;
    if (args.count()) {
      QList<QUrl> urls;
      for (int i = 0; i < args.count(); i++)
        urls.append(Utils::url(args.at(i)));
      kpl->openList(&urls, ArrayItem::Settings, 0);
    }
    kpl->show();
  }
  return application.exec();
}
