//
// C++ Implementation: newitemdlg
//
// Description: 
//
//
// Author: Werner Stille <kpl@werner-stille.de>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <QButtonGroup>
#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QGroupBox>
#include <QKeyEvent>
#include <QLayout>
#include <QPushButton>
#include <QUrl>
#include <KLocalizedString>
#include "kplitem.h"
#include "newitemdlg.h"

NewItemDlg::NewItemDlg(QWidget* parent, int* type, const QPoint& pos) :
 QDialog(parent), m_type(type), iPos(0), maxWidth(0)

{
  setWindowTitle(i18n("New item"));
  QVBoxLayout* vbox = new QVBoxLayout(this);
  QGroupBox* g = new QGroupBox;
  vbox->addWidget(g);
  g->setLayout(grid = new QGridLayout);
  bg = new QButtonGroup;
  addButton(i18n("Frame"), KplItem::Frame);
  addButton(i18n("Array"), KplItem::Array);
  addButton(i18n("Function"), KplItem::Function);
  addButton(i18n("Parametric Function"), KplItem::ParFunction);
  addButton(i18n("Spline"), KplItem::Spline);
  addButton(i18n("3D Array"), KplItem::Array3D);
  addButton(i18n("3D Function"), KplItem::Function3D);
  addButton(i18n("Image"), KplItem::Image);
  addButton(i18n("Legend"), KplItem::Legend);
  addButton(i18n("Text"), KplItem::Text);
  addButton(i18n("Line"), KplItem::Line);
  addButton(i18n("Arrow"), KplItem::Arrow);
  addButton(i18n("Arc"), KplItem::Arc);
  addButton(i18n("Rectangle"), KplItem::Rectangle);
  addButton(i18n("Ellipse"), KplItem::Ellipse);
  addButton(i18n("Scale Bar"), KplItem::ScaleBar);
  buttonBox = new QDialogButtonBox(QDialogButtonBox::Help |
                                   QDialogButtonBox::Cancel);
  vbox->addWidget(buttonBox);
  buttonBox->button(QDialogButtonBox::Cancel)->setFocus();
  connect(bg, SIGNAL(buttonClicked(int)), SLOT(slotClicked(int)));
  connect(buttonBox->button(QDialogButtonBox::Help), SIGNAL(clicked()),
          SLOT(slotHelp()));
  connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
  connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
  resize(2 * (maxWidth + 22), 100);
  move(pos);
}

NewItemDlg::~NewItemDlg()
{
}

void NewItemDlg::slotClicked(int id)
{
  *m_type = id;
  accept();
}

void NewItemDlg::slotHelp()
{
  QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/dialogs.html"
                                                "#sec-newitem")));
}

void NewItemDlg::addButton(const QString& text, int id)
{
  QPushButton* b;
  grid->addWidget(b = new QPushButton(text), iPos % 8, iPos / 8);
  maxWidth = qMax(maxWidth, b->sizeHint().width());
  bg->addButton(b, id);
  iPos++;
}

void NewItemDlg::keyPressEvent(QKeyEvent* e)
{
  if ((e->QInputEvent::modifiers() == Qt::NoModifier) &&
      (e->key() == Qt::Key_F1)) {
    buttonBox->button(QDialogButtonBox::Help)->animateClick();
    e->accept();
  } else
    QDialog::keyPressEvent(e);
}
