/***************************************************************************
                          scalebardlg.cpp  -  description
                             -------------------
    begin                : Sam Jun 14 2003
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QButtonGroup>
#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QPushButton>
#include <QRadioButton>
#include <QToolButton>
#include <QUndoStack>
#include <KLocalizedString>
#include "edittextdlg.h"
#include "kpldoc.h"
#include "scalebardlg.h"
#include "scalebaritem.h"

ScaleBarDlg::ScaleBarDlg(QWidget* parent, KplDoc* model, ScaleBarItem* si0,
                         const QPoint& pos) :
 DecoDlg(parent, i18n("Scale Bar"), model, si0, pos), si(si0)
{
  grid->addWidget(new QLabel(i18n("Orientation")), 0, 0);
  QHBoxLayout* hbox = new QHBoxLayout;
  grid->addLayout(hbox, 0, 1, 1, 5);
  hbox->addItem(new QSpacerItem(10, 10));
  bg = new QButtonGroup;
  QRadioButton* rb;
  hbox->addWidget(rb = new QRadioButton(i18n("Horizontal")));
  bg->addButton(rb, 0);
  hbox->addWidget(rb = new QRadioButton(i18n("Vertical")));
  bg->addButton(rb, 1);
  bg->button(si->vertical() ? 1 : 0)->setChecked(true);
  hbox->addItem(new QSpacerItem(10, 10, QSizePolicy::Expanding));
  grid->addWidget(new QLabel(i18n("Text")), 1, 0);
  grid->addLayout(hbox = new QHBoxLayout, 1, 1, 1, 4);
  hbox->addWidget(eText = new QLineEdit(si->text()));
  QToolButton* tb = new QToolButton;
  tb->setText("...");
  hbox->addWidget(tb);
  connect(tb, SIGNAL(clicked()), SLOT(slotEditText()));
  grid->addWidget(new QLabel(i18n("Length")), 2, 0);
  grid->addWidget(eLen = new QDoubleSpinBox, 2, 1);
  eLen->setMaximum(200);
  eLen->setSingleStep(0.1);
  eLen->setSuffix(" cm");
  eLen->setValue(si->length());
  initEntries(3);
  connect(bg, SIGNAL(buttonClicked(int)), SLOT(slotEdited()));
  connect(eLen, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eText, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(buttonBox, SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
}

ScaleBarDlg::~ScaleBarDlg()
{
}

void ScaleBarDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
      buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
      buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    case QDialogButtonBox::Ok: {
      ScaleBarItem* siNew = new ScaleBarItem(*si);
      getValues(siNew);
      siNew->setVertical(bg->checkedId());
      siNew->setText(eText->text());
      siNew->setLength(eLen->value());
      m->undoStack()->push(new EditCommand(siNew, si, m));
      si = siNew;
      if (id == QDialogButtonBox::Ok)
        accept();
      break;
    }
    case QDialogButtonBox::Help:
      QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/"
                                                    "scalebar.html")));
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}

void ScaleBarDlg::slotEditText()
{
  EditTextDlg dlg(this, eText);
  dlg.exec();
  buttonBox->button(QDialogButtonBox::Ok)->setFocus();
}
