/***************************************************************************
                          splinedlg.h  -  description
                             -------------------
    begin                : Sun Mar 3 2002
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SPLINEDLG_H
#define SPLINEDLG_H

#include <QDialog>

class QDialogButtonBox;
class QComboBox;
class KColorButton;
class BrushButton;
class DoubleEdit;
class DoubleSpinBox;
class KplDoc;
class SpinBox;
class SplineItem;
class SymbolButton;

/**
  * Spline settings dialog class. Provides a dialog for the settings
  * of a smoothing spline.
  * @author Werner Stille
  */
class SplineDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param fd pointer to spline object.
   * @param pos dialog position.
   */
  SplineDlg(QWidget* parent, KplDoc* model, SplineItem* fd, const QPoint& pos);
  /** Destructor. */
  ~SplineDlg();

protected:
  /**
   * Applies settings.
   * @param ok true to close the dialog.
   */
  void apply(bool ok = false);
  /** Copies settings to temporary spline object. */
  void getTemp();
  /**
   * Reimplemented to display help when key F1 is pressed.
   * @param e Key event.
   */
  virtual void keyPressEvent(QKeyEvent* e);
  KplDoc* m;
  SplineItem *si, *sit;
  QDialogButtonBox* buttonBox;
  QComboBox *eDeriv;
  BrushButton *fill;
  SymbolButton* bSymb;
  DoubleEdit *exLow, *exMin, *exMax, *exStep, *efx, *efy;
  DoubleSpinBox *ex0, *ey0;
  SpinBox *symb, *eRelSize;
  KColorButton* colData;

protected slots:
  /** Applies settings without leaving the dialog. */
  void slotApply();
  /**
   * Handles button clicks.
   * @param button pointer to button.
   */
  virtual void slotButtonClicked(QAbstractButton* button);
  /**
   * Enables lower boundary widget for integrals.
   * @param index index of selected derivative box item.
   */
  void slotDerivChanged(int index);
  /** Enables buttons. */
  void slotEdited();
  /** Exports to file. */
  void slotExport();
  /**
   * Adjusts state of widgets when fill pattern is changed.
   * @param brush new pattern number.
   */
  void slotFillChanged(int brush);
  /** Caculates roots. */
  void slotRoots();
};

#endif
