/***************************************************************************
                          symboldlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SYMBOLDLG_H
#define SYMBOLDLG_H

#include <QDialog>
#include <QPushButton>
#include "kplitem.h"

class QButtonGroup;
class QDialogButtonBox;
class QSpinBox;

/**
  * Symbol button. Provides a button which shows the current
  * plot symbol and opens a dialog to select markers and line
  * types.
  * @author Werner Stille
  */
class SymbolButton : public QPushButton
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param sb pointer to QSpinBox containing the symbol number.
   * @param linesOnly true to allow only lines.
   * @param disable true for disabled button and spinbox.
   * @param parent pointer to parent widget.
   */
  SymbolButton(QSpinBox* sb, bool linesOnly = false, bool disable = false,
               QWidget* parent = 0);
  /** Destructor. */
  ~SymbolButton();

protected:
  QSpinBox* symb;
  bool linesOnly;

public slots:
  /**
   * Sets pixmap on symbol selection button.
   * @param isymb symbol number.
   */
  void setButtonSymbol(int isymb);

protected slots:
  /** Displays the dialog for selection of lines and markers. */
  void slotSymbol();
};

/**
  * Symbol dialog. Provides a dialog to select markers and line types.
  * @author Werner Stille
  */
class SymbolDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param sb pointer to QSpinBox containing the symbol number.
   * @param linesOnly true to allow only lines.
   */
  SymbolDlg(QWidget* parent, QSpinBox* sb, bool linesOnly = false);
  /** Destructor. */
  virtual ~SymbolDlg();

protected:
  QSpinBox* sb;
  QButtonGroup* bg;
  QDialogButtonBox* buttonBox;
  bool linesOnly;

protected slots:
  /**
   * Handles button clicks.
   * @param button pointer to button.
   */
  virtual void slotButtonClicked(QAbstractButton* button);
};

#endif
