/***************************************************************************
                          textdlg.cpp  -  description
                             -------------------
    begin                : Wed Apr 18 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QComboBox>
#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QPushButton>
#include <QStringList>
#include <QToolButton>
#include <QUndoStack>
#include <KLocalizedString>
#include "edittextdlg.h"
#include "kpldoc.h"
#include "textdlg.h"
#include "textitem.h"

TextDlg::TextDlg(QWidget* parent, KplDoc* model, TextItem* td0,
                 const QPoint& pos) :
 DecoDlg(parent, i18n("Text"), model, td0, pos), ti(td0)
{
  grid->addWidget(new QLabel(i18n("Text")), 0, 0);
  QHBoxLayout* hbox;
  grid->addLayout(hbox = new QHBoxLayout, 0, 1, 1, 4);
  hbox->addWidget(eText = new QLineEdit(ti->text()));
  QToolButton* tb = new QToolButton;
  tb->setText("...");
  hbox->addWidget(tb);
  connect(tb, SIGNAL(clicked()), SLOT(slotEditText()));
  grid->addWidget(new QLabel(i18n("Direction")), 1, 0);
  grid->addWidget(eDir = new QDoubleSpinBox, 1, 1);
  eDir->setRange(-360, 360);
  eDir->setSuffix(QString::fromUtf8("\302\260"));
  eDir->setValue(ti->dir());
  grid->addWidget(new QLabel(i18n("Alignment")), 1, 3);
  grid->addWidget(eAlign = new QComboBox, 1, 4);
  eAlign->addItems(QStringList() << i18n("left") << i18n("centered")
                                 << i18n("right"));
  eAlign->setCurrentIndex(ti->alignment() - 1);
  initEntries(2);
  connect(eDir, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eAlign, SIGNAL(currentIndexChanged(int)), SLOT(slotEdited()));
  connect(eText, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(buttonBox, SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
}

TextDlg::~TextDlg()
{
}

void TextDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
      buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
      buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    case QDialogButtonBox::Ok: {
      TextItem* tiNew = new TextItem(*ti);
      getValues(tiNew);
      tiNew->setAlignment(TextItem::Alignment(eAlign->currentIndex() + 1));
      tiNew->setDir(eDir->value());
      tiNew->setText(eText->text());
      m->undoStack()->push(new EditCommand(tiNew, ti, m));
      ti = tiNew;
      if (id == QDialogButtonBox::Ok)
        accept();
      break;
    }
    case QDialogButtonBox::Help:
      QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/text.html")));
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}

void TextDlg::slotEditText()
{
  EditTextDlg dlg(this, eText);
  dlg.exec();
  buttonBox->button(QDialogButtonBox::Ok)->setFocus();
}
